library(dplyr)
library(tidyr)
library(ggplot2)
library(stringr)

#Set working directory to source-file location

setwd("PATH TO THIS FILE ON YOUR COMPUTER")
nomSession = "test-6-7-18/"

#Set traitement#
################

setT <- function(g) {
  if (g == 1 ) return("Model")
  if (g == 2 ) return("Puzzle")
  if (g == 3 ) return("Model")
  if (g == 4 ) return("Puzzle")
  if (g == 5 ) return("Model")
}

setTraitement <- function(vecteur) {
  return(mapply(setT, vecteur))
}


#Load data from raw files##
###########################

loadData <- function(resdir = "../results-expe-6-7-18/") {
  wat <- NULL
  plays <- NULL
  fields <- NULL
for (g in 1:5) {
wat <-  tbl_df(rbind(wat,read.table(paste0(resdir,"G1",g,"watershed.csv"), 
                         sep=",",
                         dec=".",
                         header=T)))
plays <- tbl_df(rbind(plays,read.table(paste0(resdir,"G1",g,"players.csv"),
                           sep=",",
                           dec=".",
                           header=T)))
fields <- tbl_df(rbind(fields, read.table(paste0(resdir,"G1",g,"fields.csv"), 
                            sep=",",
                            dec=".", 
                            header=T)))
}
  return(list(wat, plays, fields))
}

#Format data##
##############
formatingNSavingData <- function() {
res <- loadData()
watersheed <- res[[1]] %>%
  select(- agent., - soiled.water, - forcast) %>% 
  filter(round > 0) %>%
  distinct()
players <- res[[2]] %>%
  select(- agent.) %>% 
  filter(round > 0) %>%
  distinct()
fields <- res[[3]] %>%
  filter(round > 0) %>%
  select(- agent.) %>% 
  distinct()

players <- players %>% mutate(WAG = as.numeric(WAG),
                              round = as.numeric(round),
                              investment = as.numeric(investment))

watersheed <- watersheed %>% mutate(clean.water = as.numeric(clean.water),
                                    delivery = as.numeric(delivery),
                                    round = as.numeric(round))

fields <- fields %>% mutate(round = as.numeric(round),
                            activity = as.factor(activity),
                            field = as.factor(field)) %>%
  rename(activitySuccess = success.)

#Ajoute la carte jouée dans la table joueurs
players <- players %>% left_join(fields %>%
                                  select(- field), 
                                 by = c("i","j","player", "round"))

players <- players %>% 
  mutate(WAG = WAG - 5) %>%
  rename(profit = WAG) %>% 
  rename(groupe = j) %>% 
  select(-i) %>%
  mutate(traitement = setTraitement(groupe))%>%
  mutate(traitement = as.factor(traitement)) %>%
  mutate(player = as.factor(player)) %>%
  mutate(groupe = as.factor(groupe)) %>%
  mutate(activityNum = as.numeric(str_split(activity, pattern="", simplify = T)[,2])) %>%
  arrange(round) %>%
  group_by(player,groupe) %>%
  mutate(profit.cumule = cumsum((round > 5) * profit)) %>%
  ungroup()

write.table(players, 
            paste0(nomSession,"players.dat"),
            row.names = F,
            sep = ";",
            dec = ".")

#Create a dataset dedicated to water
water <- players %>% 
  select(round,investment, groupe) %>% 
  group_by(round, groupe)%>%
  summarise(investForage = sum(investment)) %>%
  left_join(watersheed %>% 
              rename(groupe = j) %>%
              mutate(groupe = as.factor(groupe)) %>%
              filter(source.kind == "produced") %>%
              rename(prodForage = delivery) %>%
              rename(water.left = clean.water) %>%
              select(groupe, prodForage, water.left, round),
            by = c("groupe", "round")
            )

write.table(water, 
            paste0(nomSession,"irrig-system.dat"),
            row.names = F,
            sep = ";",
            dec = ".")
}
################################################################################
############################ #Plot results# ####################################
################################################################################

players <- read.table(paste0(nomSession,"players.dat"),
                       header = T,
                       sep = ";",
                       dec = ".") %>%
  tbl_df()

water <- read.table(paste0(nomSession,"irrig-system.dat"),
                      header = T,
                      sep = ";",
                      dec = ".") %>%
  tbl_df()


## Display trajectories of player variables or actions#
#######################################################
players  %>% filter(player.name == "p2") %>% select(profit.cumule)
#Moyennes par groupe#
#####################
meanPlay <- players %>% 
  group_by(round, groupe) %>%
  mutate(activityNum = replace_na(activityNum,0)) %>%
  summarise(meanProfit = mean(profit), 
            meanContribution = mean(investment),
            meanCumulProfit = mean(profit.cumule),
            meanActivityNum = mean(activityNum))

#Investissements#
#################

players %>% 
  mutate(player = paste0("P",player)) %>%
  mutate(player = as.factor(player)) %>%
  left_join(meanPlay) %>%
  ggplot(aes(x=round,color=player)) +
  geom_line(aes(y=investment), 
            position = position_jitter( width = 0, height = 0.1),
            size = 0.7) +
  geom_line(aes(y = meanContribution, color = "Moyenne"), size= 1 ) +
  facet_wrap( ~groupe, labeller = label_both) +
  scale_x_continuous(breaks = 1:20) +
  ggtitle("Investissement des joueurs") +
  scale_color_manual(values=c("Moyenne"="black", 
                              "P1"="red","P2"="orange",
                              "P3"="brown",
                              "P4"="yellow",
                              "P5"="green")) +
  xlab("Tour de jeu") +
  ylab("Investissement forage (WAG)") +
    ggsave(paste0(nomSession , 
                  "investissements-joueurs.pdf"),
           width = 29.7, height = 21, units = "cm")

players %>% 
  select(round,investment, groupe) %>% 
  group_by(round,groupe)%>%
  summarise(contribution = sum(investment)) %>%
  ggplot(aes(x=round)) +
  geom_line(aes(y=contribution)) +
  scale_x_continuous(breaks = 1:20) +
  facet_wrap( ~groupe, labeller = label_both) +
  ggtitle("Players total investment") +
  ggsave(paste0(nomSession , 
                "investissement-total.pdf"),
         width = 29.7, height = 21, units = "cm")

#Profits#
#########

players %>% 
  mutate(player = paste0("P",player)) %>%
  mutate(player = as.factor(player)) %>%
  left_join(meanPlay) %>%
  ggplot(aes(x=round,color=player)) +
  geom_line(aes(y=profit), 
            position = position_jitter( width = 0, height = 0.1),
            size = 0.7) +
  geom_line(aes(y = meanProfit, color = "Moyenne"), size= 1 ) +
  facet_wrap( ~groupe, labeller = label_both) +
  ggtitle("Profits joueurs") +
  scale_y_continuous(breaks = -5:4) +
  scale_x_continuous(breaks = 1:20) +
  scale_color_manual(values=c("Moyenne"="black", 
                              "P1"="red","P2"="orange",
                              "P3"="brown",
                              "P4"="yellow",
                              "P5"="green")) +
  ggsave(paste0(nomSession , 
                "profits-joueurs.pdf"),
         width = 29.7, height = 21, units = "cm")

players %>% 
  mutate(player = paste0("P",player)) %>%
  mutate(player = as.factor(player)) %>%
  left_join(meanPlay) %>%
  filter(round > 5) %>%
  ggplot(aes(x=round, color=player)) +
  geom_line(aes(y=profit.cumule), 
            position = position_jitter( width = 0, height = 0.1),
            size = 0.7) +
  geom_line(aes(y = meanCumulProfit, color = "Moyenne"), size= 1 ) +
  facet_wrap( ~groupe, 
              labeller = label_both,
              scale = "free") +
  ggtitle("Profits cumulés des joueurs (échelles différentes!)") +
  scale_x_continuous(breaks = 1:20) +
  scale_color_manual(values=c("Moyenne"="black", 
                              "P1"="red","P2"="orange",
                              "P3"="brown",
                              "P4"="yellow",
                              "P5"="green")) +
  ggsave(paste0(nomSession , 
                "profits-cumul-joueurs.pdf"),
         width = 29.7, height = 21, units = "cm")

#Activités#
###########

players %>% 
  mutate(activitySuccess = factor (activitySuccess, levels= c(levels(players$activitySuccess),"no-activity"))) %>%
  mutate(activityNum = replace_na(activityNum,0)) %>%
  mutate(activitySuccess = replace_na(activitySuccess,"no-activity")) %>%
  mutate(activityNum = activityNum + (2.5 - player) / 10) %>%
  mutate(player = paste0("P",player)) %>%
  mutate(player = as.factor(player)) %>%
  left_join(meanPlay) %>%
  ggplot(aes(x=round,color=player)) +
  geom_line(aes(y=activityNum), 
            size = 0.2) +
  geom_point(aes(y=activityNum, 
                 shape = activitySuccess),
            size = 3) +
  geom_line(aes(y = meanActivityNum, color = "Moyenne"), size= 0.5 ) +
  facet_wrap( ~groupe, labeller = label_both) +
  scale_x_continuous(breaks = 1:20) +
  ggtitle("Cartes activités jouées par les joueurs") +
  scale_color_manual(values=c("Moyenne"="black", 
                              "P1"="red","P2"="orange",
                              "P3"="brown",
                              "P4"="yellow",
                              "P5"="green")) +
  xlab("Tour de jeu") +
  ylab("Numéro de carte activité (1 à 5)") +
  scale_y_continuous(breaks = 1:5) +
  ggsave(paste0(nomSession , 
                "activités-joueurs.pdf"),
         width = 29.7, height = 21, units = "cm")

## Display trajectories related to water in the irrigation system#
##################################################################

water %>% 
  gather("variable", "value",3:5) %>%
  ggplot() +
  geom_line(aes(color = variable, x = round, y = value))+
  facet_wrap( ~groupe, labeller = label_both) +
  scale_x_continuous(breaks = 1:20) +
ggtitle("Water in the game")+
  ggsave(paste0(nomSession , 
                "water-in-game.pdf"),
         width = 29.7, height = 21, units = "cm")

