#Ana-WAG User Guide

##Table of content
- [Purpose](#purpose)
- [Structure of the Ana-WAG device](#structure-of-the-ana-wag-device)
- [Setting experimental parameters (Experimenter)](#setting-experimental-parameters-experimenter)
- [Modeling (Experimenter): Design a watershed model](#modeling--experimenter-design-a-watershed-model)
- [Play (Experimenter): Organise and run a network game](#play-experimenter-organise-and-run-a-network-game)
- [Play (Player): Participate to a network game](#play-player-participate-to-a-network-game)
- [Simulate (Experimenter): Run simulations with computerized players](#simulate-experimenter-run-simulations-with-computerized-players)
- [Download and Installation of Ana-WAG](#download-and-installation-of-ana-wag)
- [Details and implementation in NetLogo](#details-and-implementation-in netlogo)
***

## Purpose
The ana-wag device, for "Analyse Wat-A-Game" (WAG), is a computer version of the Wat-A-Game "paper and pebbles" modelling and simulation tool for water management (See Abrami et al., 2012 [^1]). It enables to perform the three activities below.

1. Build up a Wat-A-Game model representing a watershed or an irrigated scheme.
2. Simulate the model by playing it as a network-game in an experimental design.
3. Simulate the model with computer agents instead of players.

The aim is to make possible to perform experiments in the understanding of contextualized experimental economics, in which subjects can build "role playing game" models as the one built during participatory processes and then play to the model they built.  In this actual version, ANA-WAG is designed to realize the specific experiment presented in a scientific paper under review[^2]. However it is already quite generic and can be easily reused for academic purposes for instance.

### Entities, state variables and scales of the WAG model
Entities of ana-wag device model correspond to the entities that exist in the  WAG role playing game and in a watershed system in general: 

- Players that represent water users, 
- Waterpaths that represent the river, 
- Fields that represent elementary spatial units, 
- Activities that represent uses of resources (water resource and eventually other resources) to produce other resources and that must be installed on a Field entity, 
- Pumps that enable to withdraw water from the river to bring it to the Fields,
- Outlets that represent the out-flow from Fields entities to the river,
- Sources of water that brings water to the Waterpaths, 
- Water resource that can flow along river path, and 
- WAG resources that represent money. 

The conceptual model of entities and their state variables is presented in an UML class diagram in Figure below.

> Ana-WAG Main Entities and state variables

>![Conceptual class diagram](images/class-concepts.png){ width=100% }

There are three main levels of spatial scale in the WAG modelling language.

- The level of the Field that is the same level than the activity in the spatial scale. It represent the elementary unit, spatially and temporally. Indeed the transformation of resource described by all Activities are processes that occur at the same level of spatial and temporal scale. This level however is not specified at this point, it depends on each WAG model. In the model used in _(Bonte et al, under rewiew)_, which is theoretical, we can consider that the spatial level is a plot of an irrigated scheme (let's say 1ha).

- The level of the set of Fields owned by a Player is the spatial scale of strategic decision making since it determines the stakes of each players. In the model used in _(Bonte et al, under rewiew)_, each Player own one or several plots.

- The level of the watershed is the greater level that contains all entities. In the model used in _(Bonte et al, under rewiew)_, it represents an irrigated scheme managed by four farmers and supplied by one natural source of water (rain from upstream) and one artificial source of water (pumped from an aquifer).

The temporal resolution is the year or the time to execute an Activity. It corresponds to a "turn" of the game. The temporal extent is the number of turns.

## Structure of the Ana-WAG device
We distinguish two kinds of users of the Ana-WAG device:

- The experimenter who can:

	* set experimental parameters,
	
	* build a watershed model, 
	
	* run a simulation with computerized agents,

	* run a game session with human players,
	
- The players who can:

	* Play a game session

## Setting experimental parameters (Experimenter)

The first feature enables the experimenter to set up the parameters of a session (group numbers, duration, water supply parameters) and to choose activity to perform (modelling, simulation or network game). The corresponding interface is the general interface displayed below opened when the file is open.

> Ana-WAG Main Inferface: Set parameters and choose activity

>![Main interface](images/main-interface.png){ width=80% }

>- To change parameters values, replace the value by the one you want and press "enter".
> - To start an activity, click on the button.

 
Parameters to set are the following:

**Define the context**

-  **i** and **j** : indexes used to save dataand relate it to a simulation or a group of players
- **nb-players**: number of players/agents in the watershed
- **initial-W**: Number of units of money at the initialisation of each round.
- **rounds**: Maximal number of rounds during a simulation or a game
- **max-investment**: Maximal possible investment in the water harvesting public infrastructure.

	
**Define the natural water parameter**

* **min natural water**: Minimum natural water in random natural sources
* **max natural water**: Maximum natural water in random natural sources
* **forcast error**: Error factor in natural water forcasting
	
**Choose activity**
See the three possible activities in next sessions:

* [Modeling (Experimenter): Design a watershed model](#modeling-experimenter-design-a-watershed-model)
* [Play (Experimenter): Organise and run a network game](#play-experimenter-organise-and-run-a-network-game)
*  [Play (Player): Participate to a network game](#play-player-participate-to-a-network-game)
* [Simulate](#simulate)


##Modeling (Experimenter): Design a watershed model
The Modeling feature enables to realise the model of an irrigated scheme model as presented in (Bonte et al., Under review[^2]). Figure below presents the interface in which an experimenter already started to draw a water shed with 4 players/ agents, a field, a river reach and  a water source (visible in the drawing area). The user may load or save his watershed and modify existing watersheds.
 
> Ana-WAG Modeling Inferface: Draw your watershed

>![Modeling interface](images/modeling-inter-guide.png){ width=110%} 

> -Click on "RUN" button to start the activity
> - Draw your watershed by drog and droping elements from the tool area to the drawing area, eventual opions will be proposed when you install elements (owner of the fields, kind of the water sources, ...).
> - Save or load your wartershed with corresponding buttons

##Play (Experimenter): Organise and run a network game
The **Play** activity enables to realise the network game model activity, organised as a client server architecture based on [HubNet](https://ccl.northwestern.edu/netlogo/hubnet.html) in which:

-  When clicking on the **Play** button, a windows open and the experimenter must first start a network session to which players will connect. He or she just has to enter a session name and click on the "start" button (see Figure below).

> Starting a network session window

>![Starting hubnet interface](images/start-hubnet-server.png)

>- Enter session name
>- Choose to broadcast session so that players can see the session when they open clients
> -Click on start

- Once the session is started the experimenter can monitor and manage the clients connexions (see interface below)

> Managing client connexions with HubNet control center interface

>![HubNet control interface](images/hubnet-server.png)

> - You can see connected clients (here one client "Bruno") and disconnect them eventually (kick button)
> - You can see server adress and port
> - You can open local client connexions (local button)
> - You can send messages to clients
>  ...

-  The experimenter manages the server interface (see Figure below) with which he can monitor players actions and decide to start, pause or resume the game. A game session si by default initialized with the default "example" watershed but you may load another existing watershed. 

> Ana-WAG Play Inferface (experimenter): Manage a game session

>![Play interface (experimenter)](images/play-inter-guide.png){ width=110%}

> - The **example** watershed model is loaded by default, if you want to use another one you need to click on load button and choose the watershed you want to play.
> - When you click on RUN button, each connected clients is associated to a  player of the watershed by order of connexion (supernumerar clients are not associated..). Here there is only one client connected (Bruno) and he is associated to red player (Player 1).
>
> - Before starting the game, you may set the speed of drops (they run thrgouh the watershed at the end of each turn) and the turn duration (3 possibilities).
> - There may be some issues with client messages sent during pauses, if this hapens, clik on "Clean server buffer" and set back the start/pause switch **On**.
> - There may be some issues in visualizing the space (all or part in gray), if it happens click on "Refresh view".

- The players are presented in next section: ([see next section](#play-experimenter-organise-and-run-a-network-game))).



##Play (Player): Participate to a network game

The players manage their client interface with which a player can at each round: monitor his own activities and status, choose his participation to the public infrastructure and change the activities to implement on his plots by clicking on a plot and choosing an activity card in the legend.

- Client must first run the HubNet client software (executable file in the root ot NetLogo installation folder that you just need to copy and paste on your computer), and connect to the server using the HubNet client connexion interface displayed below.

> Connexion with HubNet client connexion interface

> ![Client connexion interface](images/hubnet-client-connextion.png)

> - Choose the session you want to connect to (or enter manualy the server adress and port)
> - Enter a user name
> - Click on the "enter" button

- Once connected, the player waits for the game to start and then he can start playing by playing an activity card on each of his field and choosing how much he wants to contribute to the water harvesting public infrastructure. (for instance the interface of player 1 (Bruno Client) in Figure below where player 4 just changed the activity card standing on his first plot).

> Client game interface

>![Game interface client](images/game-inter-client-guide.png){ width=100%}

> - Select your investment in public water source (here player 1 choose 2) - for each one of your fields (fileds of your color) set the activity by clicking on the filed (it becomes highlited with a white halo) and then clicking on the activity you want to settle.
> - You can see how much time is remaining for the current turn (here 1 second).
> - You can see your income for previous turns (here only one previous turn t1, where your income was "-1").
> - You can see your actual wealth under the figure representing you in the top of the screen (here the red player owns 5 WAGs).

##Simulate (Experimenter): Run simulations with computerized players
The experimenter can test his watersheds models by running simulations where human players are replaced by computer agents. An inference engine has been built to model agents' behaviours but at the moment given rules are very simple so agent behariour is erratic. However it enables to watch a simulation when you do not want to play all the players (see Figure below).

> Simulation interface

>![Simulation interface](images/simu-interface.png){ width=100%}

>- Just click on RUN button and the simulation starts.
> - At beginning the game is on pause. If you want to start it, you need to click set the start/pause switch **On**.
> - Rounds are passed automaticaly and activities and investments are choosen automaticaly by each agents. 
> - You can see on the image that drops of water are flowing in the watershed.

## Download and Installation of Ana-WAG
All sofwares used are free of charge and open-source (Platform: NetLogo 5.3.1, Programming Language: NetLogo, Operating System: Platform Independent, Model code licensed Under: GNU GPL, Version 3). 
In order to use Ana-WAG you must:
- firstly download and install NetLogo 5.3.1 ([NetLogo download page](https://ccl.northwestern.edu/netlogo/download.shtml)),
- secondly download the extra-widget extension v1.1.0
([ExtraWidget extension page](https://github.com/CRESS-Surrey/eXtraWidgets/releases/tag/v1.1.0)) and install it as an extension in your NetLogo installation as explained in the [extension guide of NetLogo ](ccl.northwestern.edu/netlogo/docs/extensions.html#where-extensions-are-located) (just put the downloaded file in the rigth foler of your NetLogo install).

##Details and implementation in NetLogo
###Mzpping of the WAG model in the NetLogo world
The conceptual model of ANA-WAG presented in Figure the conceptual class diagram has been mapped to the NetLogo meta-model in order to implement the model in the NetLogo platform (See Figure below).

> Mapping of the WAG conceptual model used in ANA-WAG into the NetLogo meta-model

>![Main interface](images/class-netlogo.png){ width=100% }

###Process overview and scheduling of the WAG model
In the ana-wag device, the simulation corresponds to the network game feature or to a simulation with computer agents. When it is a network game, it is managed by a specific user of the device called the experimenter that starts each turn when all players are ready. Once the turn is started, the Players (or agents) choose the activities they want to install on each of their fields, as well as the investment they want to put in the artificial source of water. Then the Waterpath entities distribute water through each Field and the Activities entities determine if they succeed (get enough resources) or not. An UML sequence diagram presents the scheduling in  the WAG model used in ANA-WAG in Figure below.

> Sequence diagram of the WAG model used in ANA-WAG

>![Sequence diagram](images/seq-game.png){ width=80% }

[^1]: Abrami, G., Ferrand, N., Morardet, S., Murgue, C., Popova, A., De Fooij, H., Aquae-Gaudi, W., 2012. Wat-a-game, a toolkit for building role-playing games about integrated water management. In: R. Seppelt, A.A. Voinov, S. Lange, D. Bankamp (Eds.) (2012): International Environmental Modelling and Software Society (iEMSs) 2012 International Congress on Environmental Modelling and Software. Managing Resources of a Limited Planet: Pathways and Visions under Uncertainty, Sixth Biennial Meeting, Leipzig, Germany.

[^2]: Bonte et al under review.
