;;==============================================================================================================
;; Include separate files
;;==============================================================================================================

__includes["src/player_class.nls" "src/agent_class.nls" "src/waterpath_class.nls" "src/water_class.nls" "src/drop_class.nls" "src/field_class.nls" "src/activity_class.nls" "src/tank_class.nls" "src/pump_class.nls" "src/outlet_class.nls" "src/source_class.nls" "src/legend_class.nls" "src/inference_engine.nls" "src/table_class.nls"]

;;==============================================================================================================
;; Using eXtraWidget extension
;;==============================================================================================================
extensions [xw]

breed [watchers watcher]

globals
[
  ;;==============================================================================================================
  ;; water-info : list of the remaining water after the round [[forcast,available,remnant, soiled water, %]]
  ;;==============================================================================================================
  ;;
  water-info

  ;[forcast,available,remnant, soiled water, %]
  water-buffer
  ;;==============================================================================================================
  ;; initial-WAG : int
  ;;==============================================================================================================
  ;; The initial endowment in water for players
  ;;
  initial-WAG

  ;;==============================================================================================================
  ;; nb-players : int
  ;;==============================================================================================================
  ;; The number of players in the session.
  ;;
  nb-players

  ;;==============================================================================================================
  ;; agent? : boolean
  ;;==============================================================================================================
  ;; Define the kind of players (T = agent, F=human)
  ;;
  agent?

  ;;==============================================================================================================
  ;; current-session : string
  ;;==============================================================================================================
  ;; Output to show the current session "modeling" or "simulation" or "play"
  ;;
  current-session

  ;;==============================================================================================================
  ;; max-waterA : int
  ;;==============================================================================================================
  ;; total water available for public infrastructure
  ;;
  max-waterA

  ;;==============================================================================================================
  ;; min-nat-water : int
  ;;==============================================================================================================
  ;; minimu water available from natural source
  ;;
  min-nat-water

  ;;==============================================================================================================
  ;; forcast-error : float
  ;;==============================================================================================================
  ;; relative error of public water forcast
  ;;
  forcast-error

  ;;==============================================================================================================
  ;; max-nat-water : int
  ;;==============================================================================================================
  ;; maximu water available from natural source
  ;;
  max-nat-water

  ;;==============================================================================================================
  ;; wag-for-maxWA : int
  ;;==============================================================================================================
  ;; total wag to invest to get the all the produced water
  ;;
  wag-for-maxWA

  ;;==============================================================================================================
  ;; max-investment : int
  ;;==============================================================================================================
  ;; maximum wag players can invest in public infrastructure
  ;;
  max-investment

  ;;==============================================================================================================
  ;; inflection/steepness : float
  ;;==============================================================================================================
  ;; Two parameters for the production function.
  ;;
  inflection
  steepness

  ;;==============================================================================================================
  ;; current-player : int
  ;;==============================================================================================================
  ;; The number of the current player. All actions concerning a player are made to him.
  ;;
  current-player

  ;;==============================================================================================================
  ;; initial-WAG : int
  ;;==============================================================================================================
  ;; The initial endowment in water for players
  ;;
  players-wag

  ;;==============================================================================================================
  ;; legend-x : float
  ;;==============================================================================================================
  ;; xcor max for the legend.
  ;;
  legend-x

  ;;==============================================================================================================
  ;; legend-min-y : float
  ;;==============================================================================================================
  ;; ycor min for the legend.
  ;;
  legend-y


  ;;==============================================================================================================
  ;; activity-items : list of activities
  ;;==============================================================================================================
  ;; A holder for legend's item activityblock.
  ;;
  activity-items

  ;;==============================================================================================================
  ;; color-item : global color to make distinction beetween turtles
  ;;==============================================================================================================
  ;; A global variable to change color
  ;;
  color-item


  ;;==============================================================================================================
  ;; x-step : float
  ;;==============================================================================================================
  ;; step for x increase during the waterpath building.
  ;;
  x-step

  ;;==============================================================================================================
  ;; projects : list of projects
  ;;==============================================================================================================
  ;; The list of all available projects.
  ;;
  projects

  ;;==============================================================================================================
  ;; current-project : project
  ;;==============================================================================================================
  ;; The project chosen.
  ;;
  current-project

  db-activities

  ;;==============================================================================================================
  ;; flowing-resources : List of resources that flow in the watershed
  ;; fixed-resources : List of resources that are motionless
  ;;==============================================================================================================
  ;; All the ressources used in this project.

  flowing-resources

  fixed-resources

  ;;==============================================================================================================
  ;;player-state? boolean : helpful to change players' type without the setup button
  ;;==============================================================================================================
  ;; last value of agent? switch
  ;;
  player-state?

  ;;==============================================================================================================
  ;;next? : define the next time step
  ;;==============================================================================================================
  next?

  ;;==============================================================================================================
  ;;wagbank? : a boolean to tell when to pay the players
  ;;==============================================================================================================
  wagbank?

  notification?

  ;;==============================================================================================================
  ;;some globals for experiments
  ;;---------------------------
  ;;==============================================================================================================

  ;;==============================================================================================================
  ;;group: a list that will contain the traitment number i and the index of group j G(i,j)
  ;;==============================================================================================================
  group

  ;;==============================================================================================================
  ;;rounds: the number of rounds
  ;;==============================================================================================================
  rounds

  change-time?
  invest-time?

  ;;==============================================================================================================
  ;;step: the number of time step for simulation
  ;;==============================================================================================================

  step

  ;;==============================================================================================================
  ;;remain-time: Time remaining before the end of the turn (for game only)
  ;;==============================================================================================================

  remain-time

  ;;==============================================================================================================
  ;;watershed: The name of the watershed model used (netlogo world at filename watershed/watershed.nlw is loaded)
  ;;==============================================================================================================

  watershed
]

;;==============================================================================================================
;; create-drop. (observer)
;; Create a drop.
;;==============================================================================================================
to-report create-drop [ knd ld clr ]
  let my-drop []

  hatch-drops ld
  [
    set hidden? true
    set load 1
    set kind knd
    set color clr
    set my-drop lput self my-drop
  ]

  report my-drop
end

;;==============================================================================================================
;; get-total-load. (observer)
;; Compute the total load of this water.
;;==============================================================================================================
to-report get-total-load [ wtr ]

  report length [list-of-drops] of wtr

end
;;==============================================================================================================
;; get-water-for. (observer)
;; Get drops from water to create new one.
;;==============================================================================================================
to-report create-water-with [ wtr amnt ]
  let new-water nobody
  hatch-waters 1
  [
    setup-water
    set new-water self
  ]
  let new-list []

  ; shuffle the drops in the list to take n of them randomly
   ask wtr
   [
     set list-of-drops shuffle list-of-drops
   ]

   ifelse amnt <= get-total-load wtr
   [set new-list n-of amnt [list-of-drops] of wtr]
   [set new-list [list-of-drops] of wtr]


  ;; update the list of the new water.
  ask new-water [ set list-of-drops new-list ]


  ask wtr [ set list-of-drops filter [not member? ? new-list] list-of-drops ]

  report new-water

end
;;==============================================================================================================
;; mix-waters. (observer)
;; Mix two waters into one.
;;==============================================================================================================
to mix-waters [ wtr1 wtr2 ]
  ;; Mix drops in wtr1 with ones in wtr2
  show "mixing!!"
  ask wtr1 [
    let transfered-drops [list-of-drops] of wtr2
    show transfered-drops
    set list-of-drops sentence list-of-drops transfered-drops
    show [list-of-drops] of wtr1
  ]
  ;; Kill useless water
  ask wtr2 [
    show "killing water mixed with: "
    show [list-of-drops] of wtr1
    die ]
end

to get-additional-resources-for-activity [ fld ]

  let new-consumed[]
  let p-rsc nobody
  let fieldowner one-of players with [user-id = user-id-from-number [owner] of fld]
  let act nobody

  ask fieldowner
  [ set p-rsc player-resources ]


  ask fld[
    set act one-of activities in-radius 1
  ]

  if act != nobody
  [
    ask act [
      foreach fixed-resources-consumed [

        let knd-of-rsc first ?
        let match-rsc filter [ item 0 ? = knd-of-rsc ] p-rsc

        if not empty? match-rsc
        [
          ;; If there is a match resource.
          if item 1 first match-rsc > 0 and (last ? - item 1 ?) > 0
          [;; Enough resource of that kind.
            if item 1 first match-rsc >= (last ? - item 1 ?)
            [
              ;show item 1 first match-rsc

              let dif last ? - item 1 ?

              ask fieldowner
              [
                player-set-resource knd-of-rsc ((player-get-resource knd-of-rsc) - dif)
              ]

              ;;set load = needed
              set ? replace-item 1 ? last ?
            ]
          ]

        ]


        set new-consumed lput ? new-consumed

      ]
      ; set the new list of resources consumed

      set fixed-resources-consumed new-consumed
    ]

    ;;  delete the activities if there is not enough needed resources
    ask act[
      if not activity-to-be-feed
      [
        ask fld
        [
          set has-activity? false
        ]

        ask out-link-neighbors [die]
        die
      ]
    ]
  ]
end


;;==============================================================================================================
;; feed-field-with-water. (observer)
;; Feed the field.
;;==============================================================================================================
to feed-field-with-water [ wtr fld ]

  let new-consumed[]
  let act one-of activities in-radius 1

  ;get the total consumed of the activity
  let ttl-cnsmed get-total-consume act


  let water-cnsmed create-water-with wtr ttl-cnsmed


  ask act [
    foreach flowing-resources-consumed [
      let knd first ?
      let nbr length filter [[kind] of ? = knd] [list-of-drops] of water-cnsmed

      set ? replace-item 1 ? (item 1 ? + nbr)

      set new-consumed lput ? new-consumed
      ]

    ; set the new list of resources consumed
      set flowing-resources-consumed new-consumed
  ]

  ; kill the consumed water
  ask water-cnsmed [die]

  ask act [activity-update-view]

end

;;==============================================================================================================
;; Get the user-id from the number. (observer)
;; Report the user-id.
;;==============================================================================================================
to-report user-id-from-number [ num ]
  report [user-id] of one-of players with [ number = num ]
end

to-report number-from-user-id [ uid ]
  report [number] of one-of players with [ user-id = uid ]
end


;;==============================================================================================================
;; Startup the server and wait for incoming computer client.(observer)
;;==============================================================================================================
to startup

  hubnet-set-client-interface "COMPUTER" []
  ;;hubnet-reset
  experiment-session
end


;;==============================================================================================================
;; update-list-ressources. (oberserver)
;; Update the list of resources to match the current project.
;;==============================================================================================================
to update-list-ressources
  ;; Get the list of available ressources
 ; sql:exec-query word "SELECT libelle FROM kit_has_ressources k inner join `ressources` r on k.ressource_id=r.id WHERE kit_id = " (item 2 current-project)[]
 ;let ressources sql:fetch-resultset
 let ressources ["WAG" ]

  ;; Manualy enter ressources colors
  set flowing-resources []
  set fixed-resources []
  set flowing-resources lput (list ("eau propre") 84) flowing-resources
  set flowing-resources lput (list("eau sale") red) flowing-resources
  set flowing-resources lput (list("eau verte") green) flowing-resources
  set fixed-resources lput (list ("WAG") yellow ) fixed-resources
end


;;==============================================================================================================
;; Setup all the core things like database and legend's items.(observer)
;;==============================================================================================================
to setup

  init-model
  set player-state? agent?
end

to init-model
  clear-patches
  clear-drawing
  clear-output

  set next? false
  set wagbank? false

  ask turtles with [ not is-player? self ]
  [
    die
  ]

  ;; Reset the player's variables
  ask players [ die ]
  set current-player 0

  ;; Setup Legend
  set legend-x (0.1 * max-pxcor)
  set legend-y max-pycor


  set activity-items []
  set color-item 15

  ;; Set increment for Waterpath building.
  set x-step 0.5

  ;; Create the players
  create-players nb-players

  ;; init the players
  ask players
  [
    player-setup
  ]

  ;; init the group of players.
  set group[]
  set group lput xw:get "i" group
  set group lput xw:get "j" group

  ;; init the players' wag
  set players-wag[]
  ask players
  [
    set players-wag lput player-get-resource "WAG" players-wag
  ]

  ;; Show the legend
  build-legend
end


;to create-turtle-from-legend-item [a-legend-item]
;Use the data loaded at project initialisation to create a turtle corresponding to the legend selected item.
;It requires to have a global variable storing the result of the sql query (variable activities in extract-activities-from-db).

to extract-activities-from-db
;; Extract the activities from the DB.
 ;No extration in this version, the activities are manualy loaded
 ;   sql:exec-query word "SELECT activite_id, titre, ressource_id, libelle, consomme, produit FROM kit_has_activite_has_ressources k1 INNER JOIN kit_has_activites k2 ON k2.id = k1.kit_has_activite_id INNER JOIN activites a1 ON a1.id = activite_id INNER JOIN ressources r1 ON r1.id = ressource_id WHERE k2.kit_id =" (item 2 current-project) []
 ;   set db-activities sql:fetch-resultset
 set db-activities [
   [33 "A1" 1 "eau propre" 0 1]
   [34 "A2" 1 "eau propre" 0 2]
   [35 "A3" 1 "eau propre" 2 1]
   [36 "A4" 1 "eau propre" 2 0]
   [37 "A5" 1 "eau propre" 3 0]
   [40 "A0" 1 "eau propre" 0 0]
   [33 "A1" 2 "eau sale" 0 0]
   [34 "A2" 2 "eau sale" 0 0]
   [35 "A3" 2 "eau sale" 0 0]
   [36 "A4" 2 "eau sale" 0 0]
   [37 "A5" 2 "eau sale" 0 0]
   [40 "A0" 2 "eau sale" 0 0]
   [33 "A1" 4 "WAG" 1 1]
   [34 "A2" 4 "WAG" 2 1]
   [35 "A3" 4 "WAG" 1 3]
   [36 "A4" 4 "WAG" 1 4]
   [37 "A5" 4 "WAG" 2 6]
   [40 "A0" 4 "WAG" 0 0]]


end

;;==============================================================================================================
;; Get the slope of a linear function given two points of it.
;;
;; sx: xcor of the path's start
;; sy: ycor of the path's start
;; ex: xcor of the path's end
;; ey: ycor of the path's end
;;==============================================================================================================
to-report get-slope [ sx sy ex ey ] ;; observer
  ifelse (ex - sx) != 0 [
    ;; Compute the slope
    let a (ey - sy) / (ex - sx)

    report a
  ]
  [ report 0 ]
end



;;==============================================================================================================
;; Get the y-intercept of a linear function given two points of it.
;;
;; sx: xcor of the path's start
;; sy: ycor of the path's start
;; ex: xcor of the path's end
;; ey: ycor of the path's end
;;==============================================================================================================
to-report get-y-intercept [ sx sy a ] ;; observer
                                      ;; Compute the y-intercept
  report sy - a * sx
end

;;==============================================================================================================
;; Handle the user interaction with the mouse over the world.
;;==============================================================================================================
to handle-mouse-down ;; observer
  if mouse-down?
  [
    ;; Get the nearest turtlesshow
    let candidate min-one-of turtles with [ not (is-waterarea? self) and not is-drop? self] [distancexy mouse-xcor mouse-ycor]

    ;; Consider only the turtle really near
    if [distancexy mouse-xcor mouse-ycor] of candidate < 1
    [
      ;; Which legend item are we dragging ?
      ifelse  is-legend-item? candidate [ handle-legend-item candidate ]
      [
          ifelse is-tank? candidate [ handle-tank candidate ]
          [
            ifelse is-source? candidate [ handle-source candidate ]
            [
            ]
          ]
      ]
     ]
   ]
end

;;==============================================================================================================
;; Handle the user interation with the legend-item
;;==============================================================================================================
to handle-legend-item [ candidate ]

   ;; The WATCH primitive puts a "halo" around the watched turtle.
   watch candidate

   let loc-breed [item-breed] of candidate

     ask candidate
     [
       if loc-breed = "waterarea" [handle-waterpath-item ]
       if loc-breed = "source"    [handle-source-item ]
       if loc-breed = "tank"      [handle-tank-item]
       if loc-breed = "field"     [handle-field-item]
       if loc-breed = "activity"  [handle-activity-item]
       if loc-breed = "rubber"    [handle-rubber-item]

     ]
    ;; reset the perspective on kill the legend item
    reset-perspective
    ask candidate
    [ ask out-link-neighbors [die ]
      die
    ]

  ;; When the user has released the item, we build the legend.
  build-legend
end

;;==============================================================================================================
;; set the breed variable the same as the legend-item
;;==============================================================================================================
to setup-breed
  set color [color] of myself
  set shape [shape] of myself
  set heading [heading] of myself
  set size [size] of myself

  ;; hide the legend-item.
  ask myself[set hidden? true]
end

;;==============================================================================================================
;; Handle the user interaction with the clicked source. It allow the user to chose
;; the delivry of the source.
;;
;; candidate: the source clicked.
;;==============================================================================================================
to handle-source [ candidate ]
  ask candidate
  [
    source-handling
  ]
end

;;==============================================================================================================
;; Handle the user interaction with the clicked tank.
;; candidate: the tank clicked.
;;==============================================================================================================
to handle-tank [ candidate ]
  ask candidate
  [
    tank-handling
  ]
end

;;==============================================================================================================
;; Model watershed
;;==============================================================================================================

to build-watershed
    every 0.01
  [
    ;; Process user interaction with the mouse over the world.
    handle-mouse-down
    display
  ]
end

;;==============================================================================================================
;; Save the current watershed in the database or a file
;;==============================================================================================================
to save-watershed
  ;;Procedure to save watershed in a file (or to save it in the interwag DB)
  ifelse current-session = 1
  [ let file-name "no-name"
    set file-name user-input "Enter a file name! \nYour watershed will be saved in the watershed folder.\n(As a Netlogo world ascii file (.nlw extension) "
    if file-name = "" [set file-name word watershed ".nlw"]

  ifelse not (position "." file-name = false)
  [set watershed substring file-name 0 (position "." file-name)]
  [
    set watershed file-name
    set file-name word file-name ".nlw"
    ]
    export-world (word "watershed/" file-name)
    user-message (word "Your watershed has been saved as: " file-name)
  ] [
  user-message (word "You can only save watershed model in the Modeling activity.\n(Game data are automaticaly saved)")
  ]
end

;;==============================================================================================================
;; Load an existing watershed
;;==============================================================================================================
to-report load-watershed [file-name]
  ;;Procedure to load watershed from a file (or to restore it from the interwag DB)

  ifelse file-exists? (word "watershed/" file-name)
  [
    import-world (word "watershed/" file-name)
    report true
  ]
  [
    user-message (word "Watershed not found (no file: " file-name" in watershed folder")
    report false
  ]



end

;;==============================================================================================================
;; Run the game and display
;;==============================================================================================================
to run-game
  every 0.05 * 100 / speed-of-drop [
    no-display
    go]
      ;; display (and tick) causes world updates messages to be sent to clients
    ;; these types of messages should only be sent inside an every block.
    ;; otherwise, the messages will be created as fast as possible
    ;; and consume your bandwidth.
  every 0.5 [display]
     ;; hide the others' information to a player
   ;; just show information for the current player

    if running? [
      every 2 [
        clear-msg
        hide-labels
        start-new-round
      ]
    ]
 end

;;==============================================================================================================
;; GO
;;==============================================================================================================
to go
  ;show notification
  set step step + 1
  if not notification?
  [
    ask sources
    [
      ifelse source-kind = "natural"
      [source-announcement]
      [set label ""]
    ]

    set notification? false
  ]

  ;player job
    if agent? != player-state?
    [
      set current-player 0

      ask players
      [
        player-init
      ]
      set player-state? agent?
    ]

    ;; Do the jobs

     ;outlet job
    ask outlets
    [
      outlet-job
    ]

    ;;water job
    ask waters
    [
      let status water-job
      ;show onRoad
      water-flow-out
    ]

    ; tank job
    ask tanks
    [
      tank-job
    ]

    ;pump job
    ask pumps
    [
      pump-job
    ]

    ; field job
    ask fields with [has-activity?]
    [
       field-job
    ]

    ;; process incoming messages and respond to them (if needed)
    ;; listening for messages outside of the every block means that messages
    ;; get processed and responded to as fast as possible
    if ((not agent?) and (not any? waters))
    [
      listen-clients
    ]
    ;; display (and tick) causes world updates messages to be sent to clients
    ;; these types of messages should only be sent inside an every block.
    ;; otherwise, the messages will be created as fast as possible
    ;; and consume your bandwidth.
   ; every 0.4 [display]


    ;; show next time step only if we press in "start new round" button.
    if next?
    [
      tick
      set next? false
    ]


    ;; a way to pay player after the end of a time step ( looking for a better way)

    if not any? waters and wagbank? = true
    [
      if not agent?
      [
        ask players with [user-id != 0]
        [
         hubnet-reset-perspective user-id
        ]
      ]
       ;; check and pay the player if his activity is productive
      ask fields with [has-activity?]
      [
        let act activities in-radius 1

        ask act[
          activity-check-and-pay myself
        ]
      ]

      ; save the water-info and reset the buffer
      set water-info lput water-buffer water-info


      ; set the new value of players' wag in global variable players-wag
      ask players
      [
        set players-wag replace-item (number - 1) players-wag player-get-resource "WAG"
      ]

      legend-results
      ;; important to pay only one time.
      set wagbank? false
      set notification? false

      ;; if it's the last round, save the result
      if ticks = rounds
      [saving-process]

      init-water-buffer
    ]
end

to saving-process

    let file-players         (word "G"first group last group"players.csv")
    let file-watershed       (word "G"first group last group"watershed.csv")
    let file-fields          (word "G"first group last group"fields.csv")

  ;; write the files header and the initial data.
  if ticks = 0
  [
    write-in-csv-file file-players       ["i" "j" "player" "WAG" "investment" "agent?" "round" "player-name"]
    write-in-csv-file file-watershed     ["i" "j" "source-kind" "forcast" "delivery" "clean-water" "soiled-water" "agent?" "round" ]
    write-in-csv-file file-fields        ["i" "j" "field" "activity" "player" "success?" "agent?" "round"]
  ]

  ; to write in players'file
  ask players
  [
    let nline (list first group last group number player-get-resource "WAG" pi-investment agent? ticks user-id)

    write-in-csv-file file-players nline
  ]

  ; to write in sources' file
  if ticks > 0
  [
    ask sources
    [
      let nline (list first group last group source-kind forcast delivery item 2 last water-info item 3 last water-info agent? ticks )

      write-in-csv-file file-watershed nline
    ]
  ]
  ; to write in fields' file
   ask fields with [has-activity?]
  [
    let nline (list first group last group who)

    let act one-of activities in-radius 1
    set nline lput [label] of act nline

    set nline lput owner nline
    set nline lput [success?] of act nline
    set nline lput agent? nline
    set nline lput ticks nline


    write-in-csv-file file-fields nline
  ]

end

;;==============================================================================================================
;; Clean-world : acts like a garbage collector.
;;==============================================================================================================
to clean-world

  ; remove from world all inactive activities
  ask activities with [hidden?]
  [
    ask out-link-neighbors [die ]
    die
  ]


end

;;==============================================================================================================
;; Start a new round.
;;==============================================================================================================
to start-new-round
  set change-time? false
  set invest-time? false

  ifelse ticks < (rounds + 1)
  [
    ;; if a round isn't ongoing
    if not any? waters
    [
      if ticks > 0 [
       if not agent? [
         ask players with [user-id != 0][
           set history (word history " | t" ticks ":" ((player-get-resource "WAG") - 5))
           hubnet-send user-id  "Historique de profit personel" history
         ]
       ]
      ]
      ;; ask for the saving of results
      saving-process

      set next? true
      set wagbank? true


      ; restart the pumps
      ask pumps
      [
        pump-init
      ]

      ; reset the watershed
      ask activities [
        ask out-link-neighbors [die]
        die
      ]
      ask fields [set has-activity? false]
      ask players [player-init-resources]
      ask players [player-update]

      display

      ;; call the specific actions for agents or players
      let file (word "AgentRulesDatabase/maximizing profit.txt")
      if agent? and not file-exists? file
      [
        user-message (word "The rules database for the chosen agents' behavior () wasn't found")
      ]

      ; player-job: a player without WAGs cannot do actions
      ask players [set pi-investment 0]

      players-reset-perspective

      ;--------------------------------------------------------------------------------
      ; players activities change and investment
      ;--------------------------------------------------------------------------------
      if not agent?
      [

        players-change-activity-n-invest

        ; get the investment
        ask players with [user-id != 0]
        [
          player-set-resource "WAG" (player-get-resource "WAG" - pi-investment )
        ]

      ]
      ;-------------------------------------------------------------------------------
      ; end of players activities change and investment
      ;-------------------------------------------------------------------------------


      ask players with [user-id != 0 and player-get-resource "WAG" > 0]
      [
        if agent?
        [
          ;agents
          ; check the behavior database

          if file-exists? file
          [
            ; give the possibility to agent who owns a field to change his activity before next round
            if agent-is-active
            [
              while [count fields with [owner = [user-id] of myself and has-activity? = false] > 0]
              [
                agent-change-activity
                show word who "is changing activity"
              ]

              ;ask agents to invest in public infrastructure.
              agent-invest-pi
              player-set-resource "WAG" (player-get-resource "WAG" - pi-investment )
            ]
          ]
        ]
      ]

      ; clean the world : delete unused elements (old activities)
      clean-world

      ;; get the additional resources needed by the activity only if it's next round.
        ask fields with [has-activity?]
        [
          get-additional-resources-for-activity self
        ]

      ;produce water if the activity isn't a consumer.
      ask activities with [not activity-need-water?]
      [
        activity-produce-water
      ]

      ;; in each round, we ask sources to release water
      go-sources
    ]
  ]
  [
    user-message "End of game/simulation, press restart"
  ]
end

to players-reset-perspective
  if not agent? [
    ask players with [user-id != 0]
    [
      hubnet-reset-perspective user-id

      hubnet-clear-override user-id activities "hidden?"
      hubnet-clear-overrides user-id

      hubnet-send-override user-id other players "label" [""]
    ]
  ]
end

to players-change-activity-n-invest
  set change-time? true
  set invest-time? true

  ask players with [user-id != 0 and player-get-resource "WAG" > 0][
    set current-field nobody
    hubnet-send user-id  "Message" (word "Joueur " number " à vous de jouer!")
    hubnet-send user-id "Investissement" 0
  ]
  ask tables [set shape "play"]
  ;ask players with [user-id != 0]
  ;    [hubnet-reset-perspective user-id]
  display
  reset-timer
  ask players with [user-id != 0][
    hubnet-send-watch user-id one-of watchers
  ]
  let temps-restant (30 / xN)
  set remain-time temps-restant
  set skip-turn? false

  while [ (timer < (30 / xN)) and not skip-turn?]
  [
    if (floor ((30 / xN) - timer) < temps-restant) [
      set temps-restant (floor ((30 / xN) - timer))
       ask players with [user-id != 0 and player-get-resource "WAG" > 0][
      hubnet-send user-id  "Temps restant" temps-restant]
       set remain-time temps-restant
       if temps-restant = floor (20 / xN) [
           ask players with [user-id != 0][
             let uid user-id
             let fld (one-of fields with [owner = number-from-user-id uid])
             set current-field fld
             hubnet-send-watch user-id fld
         ]
       ]
      if temps-restant = 5 [ask tables [set shape "5"]]
      if temps-restant = 4 [ask tables [set shape "4"]]
      if temps-restant = 3 [ask tables [set shape "3"]]
      if temps-restant = 2 [ask tables [set shape "2"]]
      if temps-restant = 1 [ask tables [set shape "1"]]
      if temps-restant = 0 [ask tables [set shape "0"]]
      display
    ]
    listen-clients
    wait 0.05
  ]
  set skip-turn? false

  players-reset-perspective
  set change-time? false
  set invest-time? false
   ask players with [user-id != 0 and player-get-resource "WAG" > 0][
     set current-field nobody
     hubnet-send user-id  "Message" "Tour fini!"
   ]
end




to players-change-activity
  set change-time? true

  ask players with [user-id != 0 and player-get-resource "WAG" > 0][
    set current-field nobody
  ]

  reset-timer
  hubnet-send user-id  "Message" "Sélectionnez un champ et cliquez sur une activité!"
  while [ timer < 20]
  [
    ask players with [user-id != 0 and player-get-resource "WAG" > 0][
      hubnet-send user-id  "Temps restant" round (20 - timer)
      if round (20 - timer) = 5 [ask tables [set shape "5"]]
      if timer < 4 [ask tables [set shape "4"]]
      if timer < 3 [ask tables [set shape "3"]]
      if timer < 2 [ask tables [set shape "2"]]
      if timer < 1 [ask tables [set shape "1"]]
    ]
    listen-clients
    wait 0.05
  ]

  players-reset-perspective
  set change-time? false

end

to players-invest-in-pinfrast
 set invest-time? true

  reset-timer
  hubnet-send user-id "Message" "Indiquez combien de WAG vous investissez"
  while [ timer < 15]
  [
    ask players with [user-id != 0 and player-get-resource "WAG" > 0][
      hubnet-send user-id  "Temps restant" round (15 - timer)
      if timer < 5 [ask tables [set shape "5"]]
      if timer < 4 [ask tables [set shape "4"]]
      if timer < 3 [ask tables [set shape "3"]]
      if timer < 2 [ask tables [set shape "2"]]
      if timer < 1 [ask tables [set shape "1"]]
      display
    ]
    listen-clients
    wait 0.05
  ]

  set invest-time? false
  players-reset-perspective

end

;;==============================================================================================================
;; Report if a couple of cor is inside the legend.
;; x: xcor
;; y: ycor
;;==============================================================================================================
to-report is-inside-legend? [ x y ]
  report ((x >= 0 and x <= legend-x) and (y <= max-pycor)) or ((y >= 0 and y <= 0) and (x <= max-pxcor))
end

;;==============================================================================================================
;; init-water-buffer: restart the values in the water-buffer list.
;;==============================================================================================================
to init-water-buffer

  set water-buffer[]

  set water-buffer lput 0 water-buffer
  set water-buffer lput 0 water-buffer
  set water-buffer lput 0 water-buffer
  set water-buffer lput 0 water-buffer
  set water-buffer lput 0 water-buffer
end
;;==============================================================================================================
;; Activate the sources.
;;==============================================================================================================
to go-sources

  ;init-water-buffer

  let total-delivered 0
  ask sources
  [
    source-delivery
    source-announcement
    show delivery
    set total-delivered total-delivered + delivery

    source-go
  ]

  ;; add the water delivered and initialize to 0 the remaining one.
  set water-buffer replace-item 1 water-buffer total-delivered

  ask tanks
  [
    tank-go
  ]
end



;;==============================================================================================================
;; Listen for incoming client events.
;;==============================================================================================================
to listen-clients
  ;; as long as there are more messages from the clients
  ;; keep processing them.
  while [ hubnet-message-waiting? ]
  [
    ;; get the first message in the queue
    hubnet-fetch-message

     ;; New client ?
    ifelse hubnet-enter-message?
    [
      ;; Can we accept this new client ?
      ifelse count players with [ user-id != 0 ] < nb-players
      [ associate-new-player ]
      [ user-message (word "The client " hubnet-message-source " have no player left for him.") ]
    ]
    [
    ;; Command message ?
      ifelse hubnet-exit-message? ;; when clients exit we get a special message
      [ remove-player ]
      [
        ask players with [user-id = hubnet-message-source ]
          [
            execute-command hubnet-message-tag
          ] ;; otherwise the message means that the user has
      ]     ;; done something in the interface hubnet-message-tag
           ;; is the name of the widget that was changed
    ]
  ]
end

;;==============================================================================================================
;; Associates a client/agent with a player.
;;==============================================================================================================
to associate-new-player
  ask min-one-of players with [ user-id = 0  ] [ number ]
  [
    player-client-association
  ]
  players-reset-perspective
end



;;==============================================================================================================
;; Remove the association between a player and a client.
;;==============================================================================================================
to remove-player
  ask players with [user-id = hubnet-message-source]
  [
        player-remove
  ]

end



;;==============================================================================================================
;; Execute a command coming from the player interface.
;;==============================================================================================================
to execute-command [command]
  if command = "View" and change-time?
  [
    let x round item 0 hubnet-message
    let y round item 1 hubnet-message

    ifelse not is-inside-legend? x y
    [
        let fld one-of fields with [owner = number-from-user-id hubnet-message-source and distancexy x y < 3 ]
        if fld != nobody
        [
          hubnet-send-watch hubnet-message-source fld

          ask players with [user-id = hubnet-message-source]
          [ set current-field fld ]
        ]
    ]
    [
      let legitem min-one-of legend-items with [ item-breed = "activity" ] [distancexy x y]
      if legitem != nobody
      [
        let fld nobody
        ask players with [user-id = hubnet-message-source]
        [ set fld current-field ]

        if fld != nobody
        [
          ; kill the existing activity
          ask fld
          [
            if has-activity?
            [
              let act one-of activities in-radius 1

              if act != nobody
              [
                ask act
                [
                  ask out-link-neighbors [die]
                  die
                ]
                set has-activity? false

              ]
            ]
            ; create selected activity from legend
            let new-act nobody
            ask legitem [
              hatch-activities 1
              [
                setup-breed
                set shape "activity"
                set size 12
                setup-activity
                setxy [xcor] of fld [ycor] of fld
                activity-update-view
                set production? false
                set new-act self
              ]
              set hidden? false
            ]
            set has-activity? true
            ask players with [user-id != 0 and user-id != hubnet-message-source]
            [
              hubnet-send-override user-id new-act "hidden?" [ true ]
              hubnet-send-override user-id [out-link-neighbors] of new-act "hidden?" [ true ]
            ]

           every 0.1 [display]
          ]
        ]
      ]
    ]
  ]

  ; investment time

  if command = "Investissement" and invest-time?
  [
    ask players with [user-id = hubnet-message-source]
    [
      ;set the new player investment.
      let invest hubnet-message

      if player-get-resource "WAG" >= invest
      [
        ifelse invest > max-investment
        [set pi-investment max-investment]
        [set pi-investment invest]
      ]
    ]
  ]

end

;;==============================================================================================================
;; Whenever something in world changes that should be displayed in
;; a monitor on the client send the information back to the client
;;==============================================================================================================
to send-info-to-clients ;; turtle procedure
  hubnet-send user-id "location" (word "(" pxcor "," pycor ")")
end

;;==============================================================================================================
;; clear all outputs in the client interface
;;==============================================================================================================
to hubnet-clear
  hubnet-send-clear-output ["Message" "drops" "Second left"]
end

;;==============================================================================================================
;; write-in-csv-file: Write the result of the game in a file.
;; filename: the name of the file.
;; items: list of the data (or headers) to write.
;;==============================================================================================================
to write-in-csv-file [ filename items ]

  if is-list? items and not empty? items
  [
    ;; the path of the file will be defined by the experimenter.

    file-open (word "results/"filename)

    ;; quote non-numeric items
    set items map quote items
    ;; print the items
    ;; if only one item, print it.
    ifelse length items = 1
    [
      file-print first items
    ]
    [
      file-print reduce [ (word ?1 "," ?2) ] items
    ]

    ;; close-up
    file-close
  ]
end

;;==============================================================================================================
;; quote : quote non-numeric items.
;;==============================================================================================================
to-report quote [ one-item ]

  ifelse is-number? one-item
  [
    report one-item
  ]
  [
    report (word "\"" one-item "\"")
  ]
end

;;==============================================================================================================
;; getInvestment : return the total investment in public infrastructure of players.
;;==============================================================================================================
to-report getInvestment
 let y 0

 ask players[
   set y (y + pi-investment)
 ]
 report y
end
;;==============================================================================================================
;; experiment-session : allow experimenter to parameters for the current experiment session.
;;==============================================================================================================
to experiment-session
  xw:clear-all
    ;create the tab
  xw:create-tab "t0" [
    xw:set-title "Experiment parameters"
  ]
  xw:select-tab 2
  start-expe
end
;;==============================================================================================================
;; get-kits : get kits from local or distant db.
;;==============================================================================================================
to start-expe

  ; Note: This method is made to load activites cards from WAG models kits repository web site
  ; In this version we coded the activities in hard and commented all use of sql extension.
  ;---------------------------------------

  ;; Configure SQL
  ;ifelse xw:get "local?" [
    ;
  ;]
  ;[
    ;
  ;]

  ;; Get the list of available projects
  ;sql:exec-query "SELECT * FROM projets" []
  ;set projects sql:fetch-resultset
  set projects ["experiment"]

  ;xw:create-chooser "kits"[
  ;  xw:set-items map [item 3 ?] projects
  ;  xw:set-y [ xw:y + xw:height ] xw:of "local?" + 4
  ;  xw:set-width 300
  ;  xw:set-height 30
  ;
  ;  ]


  xw:create-note "n1" [
    xw:set-text "1. Define the context"
    xw:set-y 2
  ]

  ; create the text inputs i, j
  xw:create-numeric-input "i" [
      xw:set-label "G(i)"
      xw:set-value 1
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-width 40
  ]
  xw:create-numeric-input "j" [
      xw:set-label "G(j)"
      xw:set-value 1
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-x [ xw:x + xw:width ] xw:of "i" + 2
      xw:set-width 40
  ]
  xw:create-numeric-input "nb-players" [
      xw:set-label "nb-players"
      xw:set-value 4
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-x [ xw:x + xw:width ] xw:of "j" + 3
      xw:set-width 70
  ]
  xw:create-numeric-input "initial-WAG" [
      xw:set-label "initial-WAG"
      xw:set-value 5
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-x [ xw:x + xw:width ] xw:of "nb-players" + 3
      xw:set-width 70
  ]
  xw:create-numeric-input "rounds" [
      xw:set-label "rounds"
      xw:set-value 20
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-x [ xw:x + xw:width ] xw:of "initial-WAG" + 3
      xw:set-width 70
  ]
    xw:create-numeric-input "max-investment" [
      xw:set-label "max investment"
      xw:set-value 3
      xw:set-y [ xw:y + xw:height ] xw:of "n1"
      xw:set-x [ xw:x + xw:width ] xw:of "rounds" + 3
      xw:set-width 105
  ]

  xw:create-note "n2bis" [
    xw:set-text "2. Define the natural water parameters"
    xw:set-y [ xw:y + xw:height ] xw:of "initial-WAG"
    xw:set-width 400
  ]

  xw:create-numeric-input "min-nat-water" [
     xw:set-label "min natural water"
      xw:set-value 1
      xw:set-y [ xw:y + xw:height ] xw:of "n2bis"
      xw:set-width 125
  ]

  xw:create-numeric-input "max-nat-water" [
      xw:set-label "max natural water"
      xw:set-value 1
      xw:set-y [ xw:y + xw:height ] xw:of "n2bis"
      xw:set-x [ xw:x + xw:width ] xw:of "min-nat-water" + 3
      xw:set-width 125
  ]

  xw:create-numeric-input "forcast-error" [
      xw:set-label "forcast error"
      xw:set-value 0.0
      xw:set-y [ xw:y + xw:height ] xw:of "n2bis"
      xw:set-x [ xw:x + xw:width ] xw:of "max-nat-water" + 3
      xw:set-width 90
  ]

  xw:create-note "n3" [
    xw:set-text "3. Choose activity"
    xw:set-y [ xw:y + xw:height ] xw:of "forcast-error"
    xw:set-width 300
  ]
  xw:create-button "modeling" [
    xw:set-label "Modeling"
    xw:set-commands "modeling"
    xw:set-y [ xw:y + xw:height ] xw:of "n3"
    xw:set-width 90
    xw:set-color red
  ]

  xw:create-button "play" [
    xw:set-label "Play"
    xw:set-commands "play"
    xw:set-x [ xw:x + xw:width ] xw:of "modeling" + 5
    xw:set-y [ xw:y + xw:height ] xw:of "n3"
    xw:set-width 90
    xw:set-color blue
  ]

  xw:create-button "Simulate" [
    xw:set-label "Simulate"
    xw:set-commands "simulate"
    xw:set-x [ xw:x + xw:width ] xw:of "play" + 5
    xw:set-y [ xw:y + xw:height ] xw:of "n3"
    xw:set-width 90
    xw:set-color green
  ]

end
;;==============================================================================================================
;; init-treatment : set some parameters after experimenter job.
;;==============================================================================================================
to init-treatment
  clear-ticks
  set agent? false
  set notification? false
  set initial-WAG xw:get "initial-WAG"
  set nb-players  xw:get "nb-players"
  ;set max-waterA  xw:get "max-waterA"
  ;set wag-for-maxWA xw:get "wag-for-maxWA"
  ;set inflection xw:get "inflection"
  ;set steepness xw:get "steepness"
  set max-investment xw:get "max-investment"
  set max-nat-water  xw:get "max-nat-water"
  set min-nat-water  xw:get "min-nat-water"
  set forcast-error xw:get "forcast-error"
  set rounds xw:get "rounds"
  set water-info []
  init-water-buffer
  update-list-ressources
  init-model
  clear-plot
end

;;==============================================================================================================
;; modeling : process of modeling a watershed.
;;==============================================================================================================
to modeling
    set current-session 1
    set watershed "no name"
    init-treatment
    clear-output
    output-print "Modeling"
    xw:select-tab 1
end

;;==============================================================================================================
;; simulate : process of simulating with agent's behavior. (NOT USED)
;;==============================================================================================================
to simulate

  set watershed "example"
  let file-name word watershed ".nlw"

    if load-watershed file-name
    [
      build-simulation-environment
    ]
end

;;==============================================================================================================
;; build-simulation-environment: build environment for running simulations
;;==============================================================================================================
to build-simulation-environment
  reset-ticks
  set agent? true
  ask players [ player-init ]
  clear-output
  output-print "Simulation"
  xw:select-tab 1
  set current-session 2
  build-legend
end

;;==============================================================================================================
;; play: process of playing with human
;;==============================================================================================================
to play

  set watershed "example"
  let file-name word watershed ".nlw"


    ifelse load-watershed file-name
    [
      build-game-environment
    ][
  set file-name user-input "Base watershed not found, propose your own model \n (it should be a .nlw file in watershed folder)"
  ifelse not (position "." file-name = false)
  [set watershed substring file-name 0 (position "." file-name)]
  [
    set watershed file-name
    set file-name word file-name ".nlw"
    ]
  ifelse load-watershed file-name
    [
      build-game-environment
    ][
    user-message "Still not found.. Try again!"
    ]
    ]
end

;;==============================================================================================================
;; build-game-environment: build environment for playing with human
;;==============================================================================================================
to build-game-environment
      set running? false
      set current-session 3
      reset-ticks
      hubnet-reset
      clear-output
      output-print "Game"
      xw:select-tab 1

      build-legend
      set agent? false
      set notification? false
      set initial-WAG xw:get "initial-WAG"
      set nb-players  xw:get "nb-players"
      set max-investment xw:get "max-investment"
      set rounds xw:get "rounds"
      ;; init the group of players.
      set group[]
      set group lput xw:get "i" group
      set group lput xw:get "j" group
      create-tables 1
      ask tables [move-to (patch 107 65)
        set shape "table"
        set size 20
        set color 2
        set label ""]
      ask patch 112 27 [sprout-watchers 1 [
          set size 15
          set hidden? true]]
      display
end


;;==============================================================================================================
;; change-watershed: process of playing with human on a specific watershed
;;==============================================================================================================
to change-watershed

  user-message "Enter the name of your watershed model \n (it should be a .nlw file in watershed folder)"
  let file-name user-file
  set file-name substring file-name ((position "watershed/" file-name) + 10) (length file-name)
  ifelse not (position "." file-name = false)
  [set watershed substring file-name 0 (position "." file-name)]
  [
    set watershed file-name
    set file-name word file-name ".nlw"
    ]

    let session current-session
    ifelse load-watershed file-name
    [
      if session = 3 [build-game-environment]
      if session = 2 [build-simulation-environment]
      display
      if not (session = 1) [
      user-message (word "Watershed loaded!"
     "\n ATTENTION: Experimental parameters have been updated"
     "\n to modify it you must go to experimenter paramters tab, change them and reload the watershed"
     "\n - initial-WAG set to: " (xw:get "initial-WAG") "istead of: " initial-WAG " in " file-name
     "\n - max-investment set to: " (xw:get "max-investment") " instead of: " max-investment " in " file-name
     "\n - max-nat-water set to: " (xw:get "max-nat-water") " instead of: " max-nat-water " in " file-name
     "\n - min-nat-water set to: " (xw:get "min-nat-water") " instead of: " min-nat-water " in " file-name
     "\n - forcast-error set to: " (xw:get "forcast-error") " instead of: " forcast-error " in " file-name
     "\n - rounds set to: " (xw:get "rounds") " instead of: " rounds " in " file-name)
      set initial-WAG xw:get "initial-WAG"
      set max-investment xw:get "max-investment"
      set max-nat-water  xw:get "max-nat-water"
      set min-nat-water  xw:get "min-nat-water"
      set forcast-error xw:get "forcast-error"
      set rounds xw:get "rounds"
      ask players [player-init]
      ]
    ][
    user-message "Try again.."
    ]
end


;;==============================================================================================================
;; Restart: restart the model for another simulation / game
;;==============================================================================================================

to restart
  if ticks = rounds
  [
    if current-session = 2
    [simulate]
    if current-session = 3
    [play]
  ]
end

to hide-labels
if not agent?
   [
     ask players with [user-id != 0]
     [

       hubnet-send-override user-id other players "label" [""]
     ]
   ]
end

to clear-msg
    while [ hubnet-message-waiting? ]
  [
    ;; get the first message in the queue
    hubnet-fetch-message
  ]
end
@#$#@#$#@
GRAPHICS-WINDOW
260
30
1319
670
-1
-1
8.0132
1
12
1
1
1
0
0
0
1
0
130
0
75
1
1
1
round
100.0

SLIDER
20
380
125
413
speed-of-drop
speed-of-drop
1
100
99
1
1
%
HORIZONTAL

TEXTBOX
10
190
225
216
Save watershed (Modeling only)
12
0.0
1

BUTTON
20
210
190
243
Save Watershed model
save-watershed
NIL
1
T
OBSERVER
NIL
S
NIL
NIL
1

TEXTBOX
6
12
136
38
Start / Stop activity
12
0.0
1

OUTPUT
80
35
235
85
18

PLOT
20
475
220
595
players WAG
round
WAG
0.0
10.0
0.0
7.0
true
false
"" ""
PENS
"Player 1" 1.0 0 -2674135 true "" "plot item 0 players-wag"
"Player 2" 1.0 0 -955883 true "" "plot item 1 players-wag"
"Player 3 " 1.0 0 -6459832 true "" "plot item 2 players-wag"
"Player 4" 1.0 0 -1184463 true "" "if (length players-wag > 3) [plot item 3 players-wag]"
"Player 5" 1.0 0 -10899396 true "" "if (length players-wag > 4) [plot item 4 players-wag]"

TEXTBOX
15
260
190
286
Controle game (game only)
12
0.0
1

SWITCH
20
285
130
318
skip-turn?
skip-turn?
1
1
-1000

MONITOR
15
620
95
665
server ticks
step
17
1
11

SWITCH
20
330
110
363
running?
running?
0
1
-1000

SLIDER
20
425
112
458
xN
xN
1
3
3
1
1
NIL
HORIZONTAL

BUTTON
10
35
80
85
RUN
ifelse (current-session = 1 ) [\n  Build-watershed\n] [\n  run-game\n]
T
1
T
OBSERVER
NIL
G
NIL
NIL
1

MONITOR
10
130
150
175
Watershed model
watershed
17
1
11

BUTTON
155
130
217
175
Load
change-watershed
NIL
1
T
OBSERVER
NIL
L
NIL
NIL
1

TEXTBOX
5
110
225
136
Loading existing watershed
12
0.0
1

TEXTBOX
120
330
230
360
Pause at next \nround if off
12
15.0
1

TEXTBOX
135
380
285
415
Set speed of\ndrops flow
12
0.0
1

TEXTBOX
120
425
270
460
Speed factor \nfor player time
12
0.0
1

BUTTON
100
605
235
638
Clean server buffer
clear-msg
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
100
640
235
673
Refresh view
display
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
140
280
235
325
renaming time
remain-time
17
1
11

@#$#@#$#@
## WHAT IS IT?

This template contains code that can serve as a starting point for creating new HubNet activities. It shares many of the basic procedures used by other HubNet activities, which are required to connect to and communicate with clients in Disease-like activities.

## HOW IT WORKS

In activities like Disease, each client controls a single turtle on the server.  These turtles are a breed called STUDENTS.  When a client logs in we create a new student turtle and set it up with the default attributes.  Students own a variable for every widget on the client that holds a state, that is, sliders, switches, choosers, and input boxes.  Whenever a user changes one of these elements on the client, a message is sent to the server.  The server catches the message and stores the result.  In this example a slider is used to demonstrate this behavior.  You can also send messages to the client-side widgets using hubnet-send.  Monitors on clients must be updated manually by the model, that is you must send a message to a monitor every time you want the value displayed to change. For example, if you have a monitor that displays the current location of the client's avatar, you must send a message to the client like this:

     hubnet-send "location" (word xcor " " ycor)

whenever the client moves.  Buttons on the client side send but do not receive messages.  When a user presses a button, a message is sent to the server.  The server catches the message and executes the appropriate commands.  In this case, the commands should always be turtle commands since the clients control only a single turtle.

## HOW TO USE IT

To start the activity press the GO button.  Ask students to login using the HubNet client or you can test the activity locally by pressing the LOCAL button in the HubNet Control Center. To see the view in the client interface check the Mirror 2D view on clients checkbox.  The clients can use the UP, DOWN, LEFT, and RIGHT buttons to move their avatar and change the amount they move each step by changing the STEP-SIZE slider.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

0
false
0
Polygon -2674135 true false 90 75 45 75 45 150 90 150
Polygon -2674135 true false 45 75 45 45 240 45 240 75
Rectangle -2674135 true false 45 150 90 285
Rectangle -2674135 true false 45 255 240 285
Rectangle -2674135 true false 195 45 240 285

1
false
0
Polygon -2674135 true false 195 30 150 15 60 90 75 120
Rectangle -2674135 true false 150 15 195 285

2
false
0
Polygon -2674135 true false 255 75 210 75 210 150 255 150
Polygon -2674135 true false 60 75 60 45 255 45 255 75
Rectangle -2674135 true false 60 150 255 180
Rectangle -2674135 true false 60 150 105 285
Rectangle -2674135 true false 60 255 255 285

3
false
0
Polygon -2674135 true false 270 75 225 75 225 150 270 150
Polygon -2674135 true false 60 75 60 45 270 45 270 75
Rectangle -2674135 true false 60 150 270 180
Rectangle -2674135 true false 225 150 270 285
Rectangle -2674135 true false 60 255 270 285

4
false
0
Polygon -2674135 true false 255 45 210 45 60 150 105 150
Rectangle -2674135 true false 60 150 270 180
Rectangle -2674135 true false 150 150 195 285

5
false
0
Polygon -2674135 true false 105 75 60 75 60 150 105 150
Polygon -2674135 true false 60 75 60 45 270 45 270 75
Rectangle -2674135 true false 60 150 270 180
Rectangle -2674135 true false 225 150 270 285
Rectangle -2674135 true false 60 255 270 285

activity
false
0
Rectangle -16777216 true false 30 30 270 270
Rectangle -1 false false 30 30 270 270
Line -1 false 30 150 270 150
Line -2674135 false 240 120 240 195
Line -2674135 false 240 195 255 165
Line -2674135 false 240 195 225 165

agent
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 75 60 105 90
Rectangle -16777216 true false 195 60 225 90
Rectangle -16777216 true false 105 120 195 135
Rectangle -16777216 true false 45 195 255 255
Rectangle -7500403 true true 150 210 240 225
Line -16777216 false 45 180 255 180

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

box 2
false
0
Polygon -7500403 true true 150 285 270 225 270 90 150 150
Polygon -13791810 true false 150 150 30 90 150 30 270 90
Polygon -13345367 true false 30 90 30 225 150 285 150 150

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

drop
false
0
Circle -7500403 true true 73 133 152
Polygon -7500403 true true 219 181 205 152 185 120 174 95 163 64 156 37 149 7 147 166
Polygon -7500403 true true 79 182 95 152 115 120 126 95 137 64 144 37 150 6 154 165

empty circle
false
0
Circle -7500403 false true 0 0 300

empty square
false
0
Rectangle -7500403 false true 30 30 270 270

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

field
false
0
Line -7500403 true 90 45 210 45
Line -7500403 true 90 45 30 90
Line -7500403 true 30 90 30 210
Line -7500403 true 30 210 90 270
Line -7500403 true 90 270 210 270
Line -7500403 true 210 270 270 210
Line -7500403 true 270 210 270 90
Line -7500403 true 270 90 210 45
Rectangle -7500403 true true 90 255 210 285

fire department
false
0
Polygon -7500403 true true 150 55 180 60 210 75 240 45 210 45 195 30 165 15 135 15 105 30 90 45 60 45 90 75 120 60
Polygon -7500403 true true 55 150 60 120 75 90 45 60 45 90 30 105 15 135 15 165 30 195 45 210 45 240 75 210 60 180
Polygon -7500403 true true 245 150 240 120 225 90 255 60 255 90 270 105 285 135 285 165 270 195 255 210 255 240 225 210 240 180
Polygon -7500403 true true 150 245 180 240 210 225 240 255 210 255 195 270 165 285 135 285 105 270 90 255 60 255 90 225 120 240
Circle -7500403 true true 60 60 180
Circle -16777216 false false 75 75 150

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

hexagonal prism
false
0
Rectangle -7500403 true true 90 90 210 270
Polygon -1 true false 210 270 255 240 255 60 210 90
Polygon -13345367 true false 90 90 45 60 45 240 90 270
Polygon -11221820 true false 45 60 90 30 210 30 255 60 210 90 90 90

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

play
false
0
Rectangle -2674135 true false 15 45 30 180
Rectangle -2674135 true false 15 45 90 60
Rectangle -2674135 true false 75 45 90 105
Rectangle -2674135 true false 15 90 90 105
Rectangle -2674135 true false 105 45 120 180
Rectangle -2674135 true false 150 45 165 180
Rectangle -2674135 true false 150 45 225 60
Rectangle -2674135 true false 210 45 225 180
Rectangle -2674135 true false 150 105 225 120
Rectangle -2674135 true false 255 90 270 180
Polygon -2674135 true false 255 90 300 30 300 60 270 90
Polygon -2674135 true false 240 60 255 90 270 75 240 30 240 45

prodsource
false
0
Polygon -7500403 true true 150 30 210 45 210 105 90 105 90 45 150 30
Rectangle -16777216 true false 105 60 195 90
Polygon -7500403 true true 180 105 120 105 120 210 180 210
Rectangle -7500403 true true 105 120 150 180
Rectangle -7500403 true true 150 120 195 180

rain
false
0
Circle -7500403 true true 178 28 122
Circle -7500403 true true 103 13 122
Circle -7500403 true true 30 15 118
Circle -7500403 true true 0 15 88
Circle -7500403 true true 135 90 88
Circle -7500403 true true 75 90 88
Circle -7500403 true true 15 75 88
Circle -7500403 true true 58 163 32
Circle -7500403 true true 105 180 28
Circle -7500403 true true 73 193 32
Circle -7500403 true true 60 225 28

rubber
false
0
Rectangle -2674135 true false 0 60 300 240

rubber2
false
0
Rectangle -1 false false 15 75 285 225

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

table
false
0
Rectangle -7500403 true true 105 15 210 285

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

waterarea
false
0
Rectangle -7500403 true true 30 105 255 195
Rectangle -1 true false 75 135 165 165
Polygon -1 true false 165 120 165 180 210 150 165 120

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

z
false
0
Rectangle -16777216 true false 30 30 270 270
Line -1 false 30 150 270 150
Line -2674135 false 240 195 225 165
Line -2674135 false 240 195 255 165
Line -1 false 30 30 30 270
Line -1 false 30 30 270 30
Line -1 false 270 30 270 270
Line -1 false 270 270 30 270
Line -2674135 false 240 120 240 195

@#$#@#$#@
NetLogo 5.3.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="test1" repetitions="1" runMetricsEveryStep="true">
    <go>go</go>
    <metric>count players</metric>
  </experiment>
</experiments>
@#$#@#$#@
VIEW
265
155
1315
810
0
0
0
1
1
1
1
1
0
1
1
1
0
130
0
75

MONITOR
265
100
545
149
Message
NIL
0
1

MONITOR
1090
350
1207
399
Temps restant
NIL
0
1

SLIDER
1080
520
1260
553
Investissement
Investissement
0
3
0
1
1
W
HORIZONTAL

MONITOR
555
100
1315
149
Historique de profit personel
NIL
3
1

TEXTBOX
1085
495
1315
516
0      1       2       3
20
9.9
1

TEXTBOX
15
20
165
38
-
12
0.0
1

TEXTBOX
1425
960
1485
978
-
12
0.0
1

@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
